﻿using System;
using System.IO.Pipes;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace MultiConsole
{
    internal class ConsoleProxyClient
    {
        private readonly NamedPipeClientStream _pipe;

        public ConsoleProxyClient(string pipeName)
        {
            _pipe = new NamedPipeClientStream(pipeName);
            _pipe.Connect(5000);
        }

        public void Work()
        {
            var bf = new BinaryFormatter();
            var flgRuning = true;
            while (flgRuning)
            {
                var rsp = new ConsoleProxyResponse();

                ConsoleProxyCommand cmd = null;
                try
                {

                    cmd = (ConsoleProxyCommand)bf.Deserialize(_pipe);
                }
                catch (SerializationException ex)
                {
                    rsp.Success = false;
                    rsp.FailMessage = "Can not recognize command.";
                }

                if (cmd != null)
                {
                    try
                    {
                        switch (cmd.ControlCmd)
                        {
                            case ControlCmd.Shutdown:
                                flgRuning = false;
                                rsp.Success = true;
                                break;

                            case ControlCmd.Invoke:
                                rsp.Return = typeof(Console).InvokeMember(cmd.Name,
                                    BindingFlags.Public | BindingFlags.Static | BindingFlags.InvokeMethod, null, null,
                                    cmd.Param);
                                rsp.Success = true;
                                break;

                            case ControlCmd.Get:
                                rsp.Return = typeof(Console).InvokeMember(cmd.Name,
                                    BindingFlags.Public | BindingFlags.Static | BindingFlags.GetProperty, null, null, null);
                                rsp.Success = true;
                                break;

                            case ControlCmd.Set:
                                rsp.Return = typeof(Console).InvokeMember(cmd.Name,
                                    BindingFlags.Public | BindingFlags.Static | BindingFlags.SetProperty, null, null,
                                    cmd.Param);
                                rsp.Success = true;
                                break;

                            default:
                                rsp.Success = false;
                                rsp.FailMessage = "Can not recognize command.";
                                break;
                        }
                    }

                    catch (Exception ex)
                    {
                        rsp.Success = false;
                        rsp.FailMessage = ex.Message;
                    }
                }


                bf.Serialize(_pipe, rsp);
            }

            _pipe.Close();
        }
    }
}