﻿using System;
using System.Reflection;

namespace MultiConsole.InternalUtility
{
    internal class StaticWrapper: System.Dynamic.DynamicObject
    {
        private readonly Type _t;

        public StaticWrapper(Type t)
        {
            _t = t;
        }

        public override bool TryInvokeMember(System.Dynamic.InvokeMemberBinder binder, object[] args, out object result)
        {
            try
            {
                result = _t.InvokeMember(binder.Name, BindingFlags.Static | BindingFlags.Public | BindingFlags.InvokeMethod, null, null, args);
                return true;
            }
            catch
            {
                result = null;
                return false;
            }
        }

        public override bool TryGetMember(System.Dynamic.GetMemberBinder binder, out object result)
        {
            try
            {
                var p = _t.GetProperty(binder.Name);
                if (p != null)
                    result = p.GetValue(null, null);
                else
                {
                    var f = _t.GetField(binder.Name);
                    if (f != null)
                        result = f.GetValue(null);
                    else { result = null; return false; }
                }
                return true;
            }
            catch
            {
                result = null;
                return false;
            }
        }

        public override bool TrySetMember(System.Dynamic.SetMemberBinder binder, object value)
        {
            try
            {
                var p = _t.GetProperty(binder.Name);
                if (p != null)
                    p.SetValue(null, value, null);
                else
                {
                    var f = _t.GetField(binder.Name);
                    if (f != null)
                        f.SetValue(null, value);
                    else
                        return false;
                }
                return true;
            }
            catch (SystemException)
            {
                return false;
            }
        }
    }
}